@@ -26,12 +26,13 @@
 
 #include "rdkafka.h"
 
-#define FLB_KAFKA_FMT_JSON  0
-#define FLB_KAFKA_FMT_MSGP  1
-#define FLB_KAFKA_FMT_GELF  2
-#define FLB_KAFKA_BROKERS   "127.0.0.1"
-#define FLB_KAFKA_TOPIC     "fluent-bit"
-#define FLB_KAFKA_TS_KEY    "@timestamp"
+#define FLB_KAFKA_FMT_JSON            0
+#define FLB_KAFKA_FMT_MSGP            1
+#define FLB_KAFKA_FMT_GELF            2
+#define FLB_KAFKA_BROKERS             "127.0.0.1"
+#define FLB_KAFKA_TOPIC               "fluent-bit"
+#define FLB_KAFKA_TS_KEY              "@timestamp"
+#define FLB_KAFKA_QUEUE_FULL_RETRIES  10
 
 /* rdkafka log levels based on syslog(3) */
 #define FLB_KAFKA_LOG_EMERG   0
@@ -93,6 +94,8 @@ struct flb_kafka {
 
     int dynamic_topic;
 
+    int queue_full_retries;
+
     /* Internal */
     rd_kafka_t *producer;
     rd_kafka_conf_t *conf;
