@@ -190,6 +190,10 @@ flb_ctx_t *flb_create()
 /* Release resources associated to the library context */
 void flb_destroy(flb_ctx_t *ctx)
 {
+    if (!ctx) {
+        return;
+    }
+
     if (ctx->event_channel) {
         mk_event_del(ctx->event_loop, ctx->event_channel);
         flb_free(ctx->event_channel);
@@ -198,6 +202,7 @@ void flb_destroy(flb_ctx_t *ctx)
     /* Remove resources from the event loop */
     mk_event_loop_destroy(ctx->event_loop);
     flb_free(ctx);
+    ctx = NULL;
 
 #ifdef FLB_HAVE_MTRACE
     /* Stop tracing malloc and free */
@@ -478,6 +483,8 @@ static void flb_lib_worker(void *data)
     int ret;
     struct flb_config *config = data;
 
+    mk_utils_worker_rename("flb-pipeline");
+
     ret = flb_engine_start(config);
     if (ret == -1) {
         flb_engine_failed(config);
@@ -539,6 +546,18 @@ int flb_start(flb_ctx_t *ctx)
     return 0;
 }
 
+int flb_loop(flb_ctx_t *ctx)
+{
+    int ret;
+    pthread_t tid;
+
+    tid = ctx->config->worker;
+    ret = pthread_join(tid, NULL);
+    flb_debug("[lib] Fluent Bit exiting...");
+
+    return ret;
+}
+
 /* Stop the engine */
 int flb_stop(flb_ctx_t *ctx)
 {
