@@ -119,6 +119,7 @@ struct flb_parser *flb_parser_create(const char *name, const char *format,
     int len;
     int diff = 0;
     int size;
+    int is_epoch = FLB_FALSE;
     char *tmp;
     struct mk_list *head;
     struct flb_parser *p;
@@ -141,6 +142,7 @@ struct flb_parser *flb_parser_create(const char *name, const char *format,
         return NULL;
     }
     p->decoders = decoders;
+    mk_list_add(&p->_head, &config->parsers);
 
     /* Format lookup */
     if (strcasecmp(format, "regex") == 0) {
@@ -188,6 +190,10 @@ struct flb_parser *flb_parser_create(const char *name, const char *format,
         if (strstr(p->time_fmt, "%Y") || strstr(p->time_fmt, "%y")) {
             p->time_with_year = FLB_TRUE;
         }
+        else if (strstr(p->time_fmt, "%s")) {
+            is_epoch = FLB_TRUE;
+            p->time_with_year = FLB_TRUE;
+        }
         else {
             size = strlen(p->time_fmt);
             p->time_with_year = FLB_FALSE;
@@ -217,7 +223,7 @@ struct flb_parser *flb_parser_create(const char *name, const char *format,
 #else
             flb_error("[parser] timezone offset not supported");
             flb_error("[parser] you cannot use %%z/%%Z on this platform");
-            flb_free(p);
+            flb_parser_destroy(p);
             return NULL;
 #endif
         }
@@ -234,7 +240,10 @@ struct flb_parser *flb_parser_create(const char *name, const char *format,
          * - http://stackoverflow.com/questions/7114690/how-to-parse-syslog-timestamp
          * - http://code.activestate.com/lists/python-list/521885/
          */
-        if (p->time_with_year == FLB_TRUE) {
+        if (is_epoch == FLB_TRUE) {
+            tmp = strstr(p->time_fmt, "%s.%L");
+        }
+        else if (p->time_with_year == FLB_TRUE) {
             tmp = strstr(p->time_fmt, "%S.%L");
         }
         else {
@@ -275,7 +284,7 @@ struct flb_parser *flb_parser_create(const char *name, const char *format,
             len = strlen(time_offset);
             ret = flb_parser_tzone_offset(time_offset, len, &diff);
             if (ret == -1) {
-                flb_free(p);
+                flb_parser_destroy(p);
                 return NULL;
             }
             p->time_offset = diff;
@@ -289,9 +298,6 @@ struct flb_parser *flb_parser_create(const char *name, const char *format,
     p->time_keep = time_keep;
     p->types = types;
     p->types_len = types_len;
-
-    mk_list_add(&p->_head, &config->parsers);
-
     return p;
 }
 
